/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package org.log4jfugue;
import java.io.*;
import org.apache.log4j.Logger;

/**
 * Gets data to process from an existing file.
 * Uses a LogDelay class to read log4j timestamp information in the file so as
 * to pace the reading of the file to match how it was created.
 * @author btarbox
 *
 */
public class FileDataGetter extends SimpleDataGetter {
	Logger log = Logger.getLogger(FileDataGetter.class);
	String fileName;
	float speed = 1.0f;
	LogDelay logDelay = null;
	BufferedReader reader;
	
	/**
	 * Injected by Spring DI this sets the name of file to get the log data from
	 * @param fileName
	 */
	public void setFileName(String fileName)  {
		log.info("setting filename to " + fileName);
		this.fileName = fileName;

		try {
			reader = new BufferedReader(new FileReader(fileName));
		}catch(FileNotFoundException ex) {
			log.error("failed to open log file " + fileName + " ", ex);
			return;
		}
	}
	
	/**
	 * Not yet implemented, this method will allow control of the speed of the file playback,
	 * allowing for example playing back at double the recorded speed
	 */
	public void setFileDelaySpeed(float speed) {
		this.speed = speed;
	}
	
	public void setLogDelay(LogDelay logDelay) {
		this.logDelay = logDelay;
	}
	
	
	/**
	 * The player has swapped index, see if we're read more than 1 second worth of data and if
	 * so wait to let the player catch up
	 */
	public void swappedIndex() {
		if(null != logDelay) {
			logDelay.swappedIndex();
		}
	}

	
	/**
	 * The main data gathering loop, this reads data from the data file as it is
	 * available.  After each read it calls the LogDelay method to inject the 
	 * original timing to the messages
	 */
	public void run() {
		log.info("starting file data getter thread");
		this.setPriority(Thread.MIN_PRIORITY);
		
		try {
			while(reader.ready() == true){
				try {
					String data = reader.readLine();
					if(log.isDebugEnabled())
						log.debug("about to write to pipe:" + data);
					pipedWriter.write(data + "\n");
					if(null != logDelay)
						logDelay.delayProcessing(data);
				}catch(Exception ex) {
					log.error("exception ", ex);
				}
			}//while
			log.info("*** file reader got to eof ***");
			System.exit(0);
		}catch(IOException ex) {
			log.error("IOException reading or writing ", ex);
			return;
		}
	}
}
