/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package org.log4jfugue;
import java.util.*;

import org.apache.log4j.Logger;
import org.jfugue.Player;
import org.jfugue.Pattern;
import org.jfugue.Rhythm;

/**
 * Derived from the SoundBuilder class, this class builds and plays a rhythm based
 * sound.  This avoids the problem of building a chromatic string since rhythm is
 * basically atonal.
 * 
 * @author btarbox
 *
 */
public class RhythmSoundBuilder extends SoundBuilder {
	Logger log = Logger.getLogger(RhythmSoundBuilder.class);
	private MessageReceiver messageReceiver = null;
	private ArrayList<String> instrumentNames = new ArrayList<String>();
    public RhythmSoundBuilder() {    	
    }
    
    /**
     * Used by Spring DI to inject the MessageReceiver; this is primarily so that
     * this class can access the instrument definitions known by the receiver
     * @param mr
     */
    public void setMessageReceiver(MessageReceiver mr) {
    	this.messageReceiver = mr;
    	Set<String> instruments = this.messageReceiver.getInstrumentList();

    	for(String instrument : instruments) {
    		this.instrumentNames.add(instrument);
    	}
    }
    
    
    /**
     * Build the music string to send to JFugue
     */
    public void run() {
    	int index=0;
    	log.info("at RhythmSoundBuilder thread.run");
		Pattern pattern = null;
		Rhythm rhythm = new Rhythm();
		doRhythmSubstitution(rhythm);
		boolean needsClearing[] = {true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true};
    	try {
    		while(true) {
    			String allVoices = new String("");
    			music.swapIndex();
    			//Rhythm rhythm = new Rhythm();
    			for(int layerNumber=0; layerNumber < Music.MAX_MIDI_VOICES; layerNumber++){
    				InstrumentVoice voice = music.getInstrumentVoice(layerNumber);
    				if(null != voice) {
    					Integer count = music.getPlayCount(layerNumber);
    					boolean playThisLayer = true;
    					if(count == 0) { 
    						if(false == needsClearing[layerNumber]) {
    							playThisLayer = false;  // count is zero and we've already zeroed the layer
    						} else {
    							needsClearing[layerNumber] = false; // playing it now even though zero, so its cleared
    						}
    					} else {
    						needsClearing[layerNumber] = true;  // putting something in this layer so if it comes up
    					}                                       // zero next time it will need clearing
    					if(playThisLayer)
    					rhythm.setLayer(layerNumber, buildLayerString(layerNumber, count, 
    							voice.getInstrumentName() + ":"+ voice.getSearchString()  ));
    				}
    			}
    			buildMainBeat(rhythm);
    			log.info("about to play: " + index + " " + allVoices);
    			//doRhythmSubstitution(rhythm);
    			pattern = rhythm.getPattern();
    			pattern.repeat(1);
    			player.play(pattern); //returns after playing
    			player.close();  
    			log.info("back from play: " + index);
    			index++;
    		}
    	}catch(Exception ex) {
    		log.error("run method threw exception ", ex);
    		player = new Player();
    	}
    }
    
    /**
     * creates a background beat so that user hears something
     * even if no other notes are created
     * @param rhythm
     */
    private void buildMainBeat(Rhythm rhythm) {
    	rhythm.setLayer(15, "_._._._._._._.__");
    }
    
    /**
     * if count = 1, then 1 note every 16 beats
     * if count = 2  then 1 note every 8 beats
     * if count = 3  then 1 note every 5ish beats
     * if count = 4  then 1 note every 4 beats
     * so, numBeatsBetween = MEASURE_BEATS / count
     * @param layerNumber
     * @param count
     * @return
     */
    private String buildLayerString(int layerNumber, int count, String instrumentName) {
    	final char silentNote = '.';
    	final int MEASURE_BEATS = 16;  // 16 beats per measure
    	int numBeatsBetween = MEASURE_BEATS;
    	if(count > 0) {
    		numBeatsBetween = (MEASURE_BEATS / (count+1)) ; 
    	}
    	StringBuffer str = new StringBuffer();
    	int beatCount = 0;
    	for(int x=0; x < MEASURE_BEATS; x++) {
    		if(beatCount < numBeatsBetween) {
    			str.append(silentNote);
    			beatCount++;
    		}else {
    			str.append(new Integer(layerNumber).toString());
    			beatCount = 0;
    		}
    	}
    	log.info("rhythm layer " + layerNumber + " " + instrumentName +" with count:" + count + " generated:" + str);
    	return(str.toString());
    }
    
    /**
     * This is the primary smarts of the system, creating rhythm patterns from the
     * instrument counts.  AND ITS WRONG!  NEED TO GET INSTRUMENT NAMES AND VOICE NUMBERS FROM Music class
     * @param rhythm
     */ 
	private  void doRhythmSubstitution(Rhythm rhythm) {
		for(int x = 0; x < this.instrumentNames.size(); x++) {
			String substr = "[" + this.instrumentNames.get(x)+"]s";
			log.info("subst " + new Integer(x).toString().charAt(0) + " got " + substr);
			int index = music.getInstrumentIndex(this.instrumentNames.get(x));
			rhythm.addSubstitution(new Integer(index).toString().charAt(0), substr);
		}
		rhythm.addSubstitution('.', "Rs");
		rhythm.addSubstitution('_', "[ACOUSTIC_BASS_DRUM]s"); // to hold the beat
	}

/*	
 	private  void doRhythmSubstitution(Rhythm rhythm) {
		rhythm.addSubstitution('1', this.instrumentNames.get(x)   "[ACOUSTIC_BASS_DRUM]s");
		rhythm.addSubstitution('2', "[ACOUSTIC_SNARE]s");
		rhythm.addSubstitution('3', "[CLOSED_HI_HAT]s");
		rhythm.addSubstitution('4', "[OPEN_HI_HAT]s");
		rhythm.addSubstitution('5', "[ELECTRIC_SNARE]s");
		rhythm.addSubstitution('6', "[SHORT_WHISTLE]s");
		rhythm.addSubstitution('7', "[HI_WOOD_BLOCK]s");
		rhythm.addSubstitution('8', "[MARACAS]s");
		rhythm.addSubstitution('9', "[HI_BONGO]s");
		rhythm.addSubstitution('0', "[CRASH_CYMBAL_2]s");
		rhythm.addSubstitution('A', "[HAND_CLAP]s");
		rhythm.addSubstitution('B', "[CRASH_CYMBAL_1]s");
		rhythm.addSubstitution('C', "[CHINESE_CYMBAL]s");
//		rhythm.addSubstitution('D', "[SIDE_KICK]s");
//		rhythm.addSubstitution('E', "[HI_TOM]s");
		rhythm.addSubstitution('.', "Rs");
	}
*/
}