/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package org.log4jfugue;
import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.xml.DOMConfigurator;
import org.apache.log4j.net.*;
import org.log4jfugue.socketAppender.L4JFSocketNode;

/**
 * This data getter spins a thread that does socket accept and then spins another
 * thread (L4JFSocketNode) to do the actual socket reading.
 * @author btarbox
 *
 */
 public class SocketDataGetter extends SimpleDataGetter {
	static Logger log = Logger.getLogger(SocketDataGetter.class);
	
	int port = 4445;
	public SocketDataGetter() {
		log.info("at SocketDataGetter constructor");
	}
	
	/**
	 * Spring DI injects the port to use
	 * @param port
	 */
	public void setPort(int port) {
		log.info("setting port to " + port);
		this.port = port;
	}

	
	/**
	 * Get the data.  
	 */
	public void run() {
		log.info("starting SocketDataGetter thread");
		init("log4j.properties");
		
		try {
			//while(true) {
				log.info("at top of loop");
				ServerSocket serverSocket = new ServerSocket(port);
				log.info("Waiting to accept a new client.");
				Socket socket = serverSocket.accept();
				log.info("Connected to client at " + socket.getInetAddress() + " Starting new socket node.");
				Thread thread = new Thread(new L4JFSocketNode(socket, LogManager.getLoggerRepository(), pipedWriter));
				thread.start();
				log.info("started the L4JFSocketNode thread");
				thread.join();
				log.info("The L4JFSocketNode thread exited");
			//}
		}catch(Exception ex) {
			log.error("exception ", ex);
		}
	}
	static void init(String configFile) {
		log.info("about to PropertyConfigurator " + configFile);
		if(configFile.endsWith(".xml")) {
			new DOMConfigurator().configure(configFile);
		} else {
			new PropertyConfigurator().configure(configFile);
		}
	}
}
