/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

package org.log4jfugue.socketAppender;

import java.net.Socket;
import java.net.ServerSocket;

import org.apache.log4j.Category;
import org.apache.log4j.LogManager;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.xml.DOMConfigurator;


/**
 *  A simple {@link SocketNode} based server.
 *
      Usage: java org.apache.log4j.net.SimpleSocketServer port configFile

   where port is a part number where the server listens and
   configFile is a configuration file fed to the {@link
   PropertyConfigurator} or to {@link DOMConfigurator} if an XML file.
   
  *
  * @author  Ceki Glc
  *
  *  @since 0.8.4 
  * */
public class SimpleSocketServer  {

	static Category cat = Category.getInstance(SimpleSocketServer.class.getName());

	static int port;

	public static void main(String argv[]) {
		if(argv.length == 2) {
			init(argv[0], argv[1]);
		} else {
			usage("Wrong number of arguments.");
		}

		try {
			cat.info("Listening on port " + port);
			ServerSocket serverSocket = new ServerSocket(port);
			while(true) {
				cat.info("Waiting to accept a new client.");
				Socket socket = serverSocket.accept();
				cat.info("Connected to client at " + socket.getInetAddress());
				cat.info("Starting new socket node.");
				new Thread(new L4JFSocketNode(socket,
						LogManager.getLoggerRepository(), null)).start();
			}
		} catch(Exception e) {
			e.printStackTrace();
		}
	}


	static void  usage(String msg) {
		System.err.println(msg);
		System.err.println(
				"Usage: java " +SimpleSocketServer.class.getName() + " port configFile");
		System.exit(1);
	}

	static void init(String portStr, String configFile) {
		try {
			port = Integer.parseInt(portStr);
		} catch(java.lang.NumberFormatException e) {
			e.printStackTrace();
			usage("Could not interpret port number ["+ portStr +"].");
		}

		if(configFile.endsWith(".xml")) {
			new DOMConfigurator().configure(configFile);
		} else {
			new PropertyConfigurator().configure(configFile);
		}
	}
}
